@@ -85,6 +85,11 @@ lilv_node_new(LilvWorld* world, LilvNodeType type, const char* str)
 		break;
 	}
 
+	if (!val->node) {
+		free(val);
+		return NULL;
+	}
+
 	return val;
 }
 
@@ -92,6 +97,10 @@ lilv_node_new(LilvWorld* world, LilvNodeType type, const char* str)
 LilvNode*
 lilv_node_new_from_node(LilvWorld* world, const SordNode* node)
 {
+	if (!node) {
+		return NULL;
+	}
+
 	LilvNode*    result       = NULL;
 	SordNode*    datatype_uri = NULL;
 	LilvNodeType type         = LILV_VALUE_STRING;
