@@ -20,6 +20,7 @@
 #include <stdlib.h>
 #include <string.h>
 
+#include "lv2/lv2plug.in/ns/ext/atom/atom.h"
 #include "lv2/lv2plug.in/ns/ext/event/event.h"
 
 #include "lilv_internal.h"
@@ -31,7 +32,7 @@ lilv_port_new(LilvWorld*      world,
               const char*     symbol)
 {
 	LilvPort* port = (LilvPort*)malloc(sizeof(LilvPort));
-	port->node    = sord_node_copy(node);
+	port->node    = lilv_node_new_from_node(world, node);
 	port->index   = index;
 	port->symbol  = lilv_node_new(world, LILV_VALUE_STRING, symbol);
 	port->classes = lilv_nodes_new();
@@ -42,7 +43,7 @@ void
 lilv_port_free(const LilvPlugin* plugin, LilvPort* port)
 {
 	if (port) {
-		sord_node_free(plugin->world->world, port->node);
+		lilv_node_free(port->node);
 		lilv_nodes_free(port->classes);
 		lilv_node_free(port->symbol);
 		free(port);
@@ -68,16 +69,11 @@ lilv_port_has_property(const LilvPlugin* p,
                        const LilvPort*   port,
                        const LilvNode*   property)
 {
-	assert(property);
-	SordIter* results = lilv_world_query_internal(
+	return lilv_world_ask_internal(
 		p->world,
-		port->node,
+		port->node->node,
 		p->world->uris.lv2_portProperty,
 		lilv_node_as_node(property));
-
-	const bool ret = !sord_iter_end(results);
-	sord_iter_free(results);
-	return ret;
 }
 
 LILV_API
@@ -86,16 +82,19 @@ lilv_port_supports_event(const LilvPlugin* p,
                          const LilvPort*   port,
                          const LilvNode*   event)
 {
-	assert(event);
-	SordIter* results = lilv_world_query_internal(
-		p->world,
-		port->node,
-		sord_new_uri(p->world->world, (const uint8_t*)LV2_EVENT__supportsEvent),
-		lilv_node_as_node(event));
+	const uint8_t* predicates[] = { (const uint8_t*)LV2_EVENT__supportsEvent,
+	                                (const uint8_t*)LV2_ATOM__supports,
+	                                NULL };
 
-	const bool ret = !sord_iter_end(results);
-	sord_iter_free(results);
-	return ret;
+	for (const uint8_t** pred = predicates; *pred; ++pred) {
+		if (lilv_world_ask_internal(p->world,
+		                            port->node->node,
+		                            sord_new_uri(p->world->world, *pred),
+		                            lilv_node_as_node(event))) {
+			return true;
+		}
+	}
+	return false;
 }
 
 static LilvNodes*
@@ -103,15 +102,19 @@ lilv_port_get_value_by_node(const LilvPlugin* p,
                             const LilvPort*   port,
                             const SordNode*   predicate)
 {
-	assert(sord_node_get_type(predicate) == SORD_URI);
-
-	SordIter* results = lilv_world_query_internal(
+	return lilv_world_find_nodes_internal(
 		p->world,
-		port->node,
+		port->node->node,
 		predicate,
 		NULL);
+}
 
-	return lilv_nodes_from_stream_objects(p->world, results, SORD_OBJECT);
+LILV_API
+const LilvNode*
+lilv_port_get_node(const LilvPlugin* plugin,
+                   const LilvPort*   port)
+{
+	return port->node;
 }
 
 LILV_API
@@ -131,6 +134,21 @@ lilv_port_get_value(const LilvPlugin* p,
 		lilv_node_as_node(predicate));
 }
 
+LILV_API
+LilvNode*
+lilv_port_get(const LilvPlugin* p,
+              const LilvPort*   port,
+              const LilvNode*   predicate)
+{
+	LilvNodes* values = lilv_port_get_value(p, port, predicate);
+
+	LilvNode* value = lilv_node_duplicate(
+		values ? lilv_nodes_get_first(values) : NULL);
+
+	lilv_nodes_free(values);
+	return value;
+}
+
 LILV_API
 uint32_t
 lilv_port_get_index(const LilvPlugin* p,
@@ -219,7 +237,7 @@ lilv_port_get_scale_points(const LilvPlugin* p,
 {
 	SordIter* points = lilv_world_query_internal(
 		p->world,
-		port->node,
+		port->node->node,
 		sord_new_uri(p->world->world, (const uint8_t*)LV2_CORE__scalePoint),
 		NULL);
 
