@@ -170,6 +170,7 @@ struct TestCase {
 	TestFunc func;
 };
 
+#define PREFIX_ATOM "@prefix atom: <http://lv2plug.in/ns/ext/atom#> . \n"
 #define PREFIX_LINE "@prefix : <http://example.org/> .\n"
 #define PREFIX_LV2 "@prefix lv2: <http://lv2plug.in/ns/lv2core#> .\n"
 #define PREFIX_LV2EV "@prefix lv2ev: <http://lv2plug.in/ns/ext/event#> . \n"
@@ -180,7 +181,7 @@ struct TestCase {
 #define PREFIX_DOAP "@prefix doap: <http://usefulinc.com/ns/doap#> .\n"
 
 #define MANIFEST_PREFIXES PREFIX_LINE PREFIX_LV2 PREFIX_RDFS
-#define BUNDLE_PREFIXES PREFIX_LINE PREFIX_LV2 PREFIX_RDF PREFIX_RDFS PREFIX_FOAF PREFIX_DOAP
+#define BUNDLE_PREFIXES PREFIX_ATOM PREFIX_LINE PREFIX_LV2 PREFIX_RDF PREFIX_RDFS PREFIX_FOAF PREFIX_DOAP
 #define PLUGIN_NAME(name) "doap:name \"" name "\""
 #define LICENSE_GPL "doap:license <http://usefulinc.com/doap/licenses/gpl>"
 
@@ -561,6 +562,14 @@ test_plugin(void)
 	TEST_ASSERT(!strcmp(lilv_node_as_string(klass_uri),
 			"http://lv2plug.in/ns/lv2core#CompressorPlugin"));
 
+	LilvNode* rdf_type = lilv_new_uri(
+		world, "http://www.w3.org/1999/02/22-rdf-syntax-ns#type");
+	TEST_ASSERT(lilv_world_ask(world,
+	                           lilv_plugin_get_uri(plug),
+	                           rdf_type,
+	                           klass_uri));
+	lilv_node_free(rdf_type);
+
 	const LilvNode* plug_bundle_uri = lilv_plugin_get_bundle_uri(plug);
 	TEST_ASSERT(!strcmp(lilv_node_as_string(plug_bundle_uri), bundle_dir_uri));
 
@@ -616,7 +625,7 @@ test_plugin(void)
 
 	LilvNode* lv2_latency = lilv_new_uri(world,
 			"http://lv2plug.in/ns/lv2core#latency");
-	LilvPort* latency_port = lilv_plugin_get_port_by_designation(
+	const LilvPort* latency_port = lilv_plugin_get_port_by_designation(
 		plug, out_class, lv2_latency);
 	lilv_node_free(lv2_latency);
 
@@ -702,6 +711,8 @@ test_plugin(void)
 	TEST_ASSERT(thing_name);
 	TEST_ASSERT(lilv_node_is_string(thing_name));
 	TEST_ASSERT(!strcmp(lilv_node_as_string(thing_name), "Something else"));
+	LilvNode* thing_name2 = lilv_world_get(world, thing_uri, name_p, NULL);
+	TEST_ASSERT(lilv_node_equals(thing_name, thing_name2));
 
 	LilvUIs* uis = lilv_plugin_get_uis(plug);
 	TEST_ASSERT(lilv_uis_size(uis) == 0);
@@ -720,6 +731,7 @@ test_plugin(void)
 
 	lilv_nodes_free(thing_names);
 	lilv_node_free(thing_uri);
+	lilv_node_free(thing_name2);
 	lilv_node_free(name_p);
 	lilv_node_free(control_class);
 	lilv_node_free(audio_class);
@@ -756,7 +768,16 @@ test_port(void)
 			"  a lv2:EventPort ; a lv2:InputPort ; "
 			"  lv2:index 1 ; lv2:symbol \"event_in\" ; "
 			"  lv2:name \"Event Input\" ; "
-     		"  lv2ev:supportsEvent <http://example.org/event> "
+     		"  lv2ev:supportsEvent <http://example.org/event> ;"
+     		"  atom:supports <http://example.org/atomEvent> "
+			"] , [\n"
+			"  a lv2:AudioPort ; a lv2:InputPort ; "
+			"  lv2:index 2 ; lv2:symbol \"audio_in\" ; "
+			"  lv2:name \"Audio Input\" ; "
+			"] , [\n"
+			"  a lv2:AudioPort ; a lv2:OutputPort ; "
+			"  lv2:index 3 ; lv2:symbol \"audio_out\" ; "
+			"  lv2:name \"Audio Output\" ; "
 			"] ."))
 		return 0;
 
@@ -784,9 +805,11 @@ test_port(void)
 			"http://lv2plug.in/ns/lv2core#ControlPort");
 	LilvNode* in_class = lilv_new_uri(world,
 			"http://lv2plug.in/ns/lv2core#InputPort");
+	LilvNode* out_class = lilv_new_uri(world,
+			"http://lv2plug.in/ns/lv2core#OutputPort");
 
 	TEST_ASSERT(lilv_nodes_size(lilv_port_get_classes(plug, p)) == 2);
-	TEST_ASSERT(lilv_plugin_get_num_ports(plug) == 2);
+	TEST_ASSERT(lilv_plugin_get_num_ports(plug) == 4);
 	TEST_ASSERT(lilv_port_is_a(plug, p, control_class));
 	TEST_ASSERT(lilv_port_is_a(plug, p, in_class));
 	TEST_ASSERT(!lilv_port_is_a(plug, p, audio_class));
@@ -839,6 +862,9 @@ test_port(void)
 	LilvNodes* comments     = lilv_port_get_value(plug, p, rdfs_comment);
 	TEST_ASSERT(!strcmp(lilv_node_as_string(lilv_nodes_get_first(comments)),
 	                    "comment"));
+	LilvNode* comment = lilv_port_get(plug, p, rdfs_comment);
+	TEST_ASSERT(!strcmp(lilv_node_as_string(comment), "comment"));
+	lilv_node_free(comment);
 	lilv_nodes_free(comments);
 
 	setenv("LANG", "fr", 1);
@@ -906,8 +932,10 @@ test_port(void)
 
 	LilvNode* event_type = lilv_new_uri(world, "http://example.org/event");
 	LilvNode* event_type_2 = lilv_new_uri(world, "http://example.org/otherEvent");
+	LilvNode* atom_event = lilv_new_uri(world, "http://example.org/atomEvent");
 	TEST_ASSERT(lilv_port_supports_event(plug, ep, event_type));
 	TEST_ASSERT(!lilv_port_supports_event(plug, ep, event_type_2));
+	TEST_ASSERT(lilv_port_supports_event(plug, ep, atom_event));
 
 	LilvNode* name_p = lilv_new_uri(world, "http://lv2plug.in/ns/lv2core#name");
 	LilvNodes* names = lilv_port_get_value(plug, p, name_p);
@@ -934,6 +962,25 @@ test_port(void)
 	TEST_ASSERT(lilv_nodes_size(names) == 1);
 	TEST_ASSERT(!strcmp(lilv_node_as_string(lilv_nodes_get_first(names)),
 	                    "Event Input"));
+
+	const LilvPort* ap_in = lilv_plugin_get_port_by_index(plug, 2);
+
+	TEST_ASSERT(lilv_port_is_a(plug, ap_in, in_class));
+	TEST_ASSERT(!lilv_port_is_a(plug, ap_in, out_class));
+	TEST_ASSERT(lilv_port_is_a(plug, ap_in, audio_class));
+	TEST_ASSERT(!lilv_port_is_a(plug, ap_in, control_class));
+
+	const LilvPort* ap_out = lilv_plugin_get_port_by_index(plug, 3);
+
+	TEST_ASSERT(lilv_port_is_a(plug, ap_out, out_class));
+	TEST_ASSERT(!lilv_port_is_a(plug, ap_out, in_class));
+	TEST_ASSERT(lilv_port_is_a(plug, ap_out, audio_class));
+	TEST_ASSERT(!lilv_port_is_a(plug, ap_out, control_class));
+
+	TEST_ASSERT(lilv_plugin_get_num_ports_of_class(plug, control_class, in_class , NULL) == 1);
+	TEST_ASSERT(lilv_plugin_get_num_ports_of_class(plug, audio_class  , in_class , NULL) == 1);
+	TEST_ASSERT(lilv_plugin_get_num_ports_of_class(plug, audio_class  , out_class, NULL) == 1);
+
 	lilv_nodes_free(names);
 	lilv_node_free(name_p);
 
@@ -941,6 +988,7 @@ test_port(void)
 	lilv_node_free(toggled_prop);
 	lilv_node_free(event_type);
 	lilv_node_free(event_type_2);
+	lilv_node_free(atom_event);
 
 	lilv_node_free(min);
 	lilv_node_free(max);
@@ -1106,9 +1154,9 @@ set_port_value(const char*     port_symbol,
                uint32_t        type)
 {
 	if (!strcmp(port_symbol, "input")) {
-		in = *(float*)value;
+		in = *(const float*)value;
 	} else if (!strcmp(port_symbol, "output")) {
-		out = *(float*)value;
+		out = *(const float*)value;
 	} else {
 		fprintf(stderr, "error: set_port_value for nonexistent port `%s'\n",
 		        port_symbol);
@@ -1214,6 +1262,15 @@ test_state(void)
 	// Ensure they are equal
 	TEST_ASSERT(lilv_state_equals(state, state2));
 
+	// Check that we can't save a state with no URI
+	char* bad_state_str = lilv_state_to_string(
+		world, &map, &unmap, state, NULL, NULL);
+	TEST_ASSERT(!bad_state_str);
+
+	// Check that we can't restore the NULL string (and it doesn't crash)
+	LilvState* bad_state = lilv_state_new_from_string(world, &map, NULL);
+	TEST_ASSERT(!bad_state);
+	
 	// Save state to a string
 	char* state1_str = lilv_state_to_string(
 		world, &map, &unmap, state, "http://example.org/state1", NULL);
@@ -1257,23 +1314,23 @@ test_state(void)
 
 	// Save state to a directory
 	int ret = lilv_state_save(world, &map, &unmap, state, NULL,
-	                          "state.lv2", "state.ttl");
+	                          "state/state.lv2", "state.ttl");
 	TEST_ASSERT(!ret);
 
 	// Load state from directory
 	LilvState* state5 = lilv_state_new_from_file(world, &map, NULL,
-	                                             "state.lv2/state.ttl");
+	                                             "state/state.lv2/state.ttl");
 
 	TEST_ASSERT(lilv_state_equals(state, state5));  // Round trip accuracy
 
 	// Save state with URI to a directory
 	const char* state_uri = "http://example.org/state";
 	ret = lilv_state_save(world, &map, &unmap, state, state_uri,
-	                      "state6.lv2", "state6.ttl");
+	                      "state/state6.lv2", "state6.ttl");
 	TEST_ASSERT(!ret);
 
 	// Load default bundle into world and load state from it
-	uint8_t*  state6_path       = (uint8_t*)lilv_path_absolute("state6.lv2/");
+	uint8_t*  state6_path       = (uint8_t*)lilv_path_absolute("state/state6.lv2/");
 	SerdNode  state6_uri        = serd_node_new_file_uri(state6_path, 0, 0, true);
 	LilvNode* test_state_bundle = lilv_new_uri(world, (const char*)state6_uri.buf);
 	LilvNode* test_state_node   = lilv_new_uri(world, state_uri);
@@ -1312,13 +1369,13 @@ test_state(void)
 	// Get instance state state
 	LilvState* fstate = lilv_state_new_from_instance(
 		plugin, instance, &map,
-		file_dir, copy_dir, link_dir, "fstate.lv2",
+		file_dir, copy_dir, link_dir, "state/fstate.lv2",
 		get_port_value, world, 0, ffeatures);
 
 	// Get another instance state
 	LilvState* fstate2 = lilv_state_new_from_instance(
 		plugin, instance, &map,
-		file_dir, copy_dir, link_dir, "fstate2.lv2",
+		file_dir, copy_dir, link_dir, "state/fstate2.lv2",
 		get_port_value, world, 0, ffeatures);
 
 	// Should be identical
@@ -1329,7 +1386,7 @@ test_state(void)
 
 	// Get yet another instance state
 	LilvState* fstate3 = lilv_state_new_from_instance(
-		plugin, instance, &map, file_dir, copy_dir, link_dir, "fstate3.lv2",
+		plugin, instance, &map, file_dir, copy_dir, link_dir, "state/fstate3.lv2",
 		get_port_value, world, 0, ffeatures);
 
 	// Should be different
@@ -1337,12 +1394,12 @@ test_state(void)
 
 	// Save state to a directory
 	ret = lilv_state_save(world, &map, &unmap, fstate, NULL,
-	                      "fstate.lv2", "fstate.ttl");
+	                      "state/fstate.lv2", "fstate.ttl");
 	TEST_ASSERT(!ret);
 
 	// Load state from directory
 	LilvState* fstate4 = lilv_state_new_from_file(world, &map, NULL,
-	                                              "fstate.lv2/fstate.ttl");
+	                                              "state/fstate.lv2/fstate.ttl");
 	TEST_ASSERT(lilv_state_equals(fstate, fstate4));  // Round trip accuracy
 
 	// Restore instance state to loaded state
@@ -1351,18 +1408,18 @@ test_state(void)
 	// Take a new snapshot and ensure it matches
 	LilvState* fstate5 = lilv_state_new_from_instance(
 		plugin, instance, &map,
-		file_dir, copy_dir, link_dir, "fstate5.lv2",
+		file_dir, copy_dir, link_dir, "state/fstate5.lv2",
 		get_port_value, world, 0, ffeatures);
 	TEST_ASSERT(lilv_state_equals(fstate3, fstate5));
 
 	// Save state to a (different) directory again
 	ret = lilv_state_save(world, &map, &unmap, fstate, NULL,
-	                      "fstate6.lv2", "fstate6.ttl");
+	                      "state/fstate6.lv2", "fstate6.ttl");
 	TEST_ASSERT(!ret);
 
 	// Reload it and ensure it's identical to the other loaded version
 	LilvState* fstate6 = lilv_state_new_from_file(world, &map, NULL,
-	                                              "fstate6.lv2/fstate6.ttl");
+	                                              "state/fstate6.lv2/fstate6.ttl");
 	TEST_ASSERT(lilv_state_equals(fstate4, fstate6));
 
 	// Run, changing rec file (without changing size)
@@ -1371,18 +1428,18 @@ test_state(void)
 	// Take a new snapshot
 	LilvState* fstate7 = lilv_state_new_from_instance(
 		plugin, instance, &map,
-		file_dir, copy_dir, link_dir, "fstate7.lv2",
+		file_dir, copy_dir, link_dir, "state/fstate7.lv2",
 		get_port_value, world, 0, ffeatures);
 	TEST_ASSERT(!lilv_state_equals(fstate6, fstate7));
 
 	// Save the changed state to a (different) directory again
 	ret = lilv_state_save(world, &map, &unmap, fstate7, NULL,
-	                      "fstate7.lv2", "fstate7.ttl");
+	                      "state/fstate7.lv2", "fstate7.ttl");
 	TEST_ASSERT(!ret);
 
 	// Reload it and ensure it's changed
 	LilvState* fstate72 = lilv_state_new_from_file(world, &map, NULL,
-	                                              "fstate7.lv2/fstate7.ttl");
+	                                              "state/fstate7.lv2/fstate7.ttl");
 	TEST_ASSERT(lilv_state_equals(fstate72, fstate7));
 	TEST_ASSERT(!lilv_state_equals(fstate6, fstate72));
 
